package article_test

import (
	"math"
	"reflect"
	"testing"

	article "gitlab.com/endeworks/gihyo-wdpress/when-in-go/21-Formatting-Values/src"
)

func TestVarious(t *testing.T) {
	// 様々なフォーマット指定子の差を見ます
	tests := []struct {
		Format string
		Value  interface{}
	}{
		{
			Format: `%d`,
			Value:  1,
		},
		{
			Format: `%f`,
			Value:  math.Pi,
		},
		{
			Format: `%s`,
			Value:  "Hello",
		},
		{
			Format: `%s`,
			Value:  &struct{ Foo string }{Foo: "foo"},
		},
		{
			Format: `%s`,
			Value:  [16]byte{0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF, 0x0},
		},
	}

	for _, tc := range tests {
		t.Logf(tc.Format+` (%s)`, tc.Value, tc.Format)
		t.Logf(`%v (%%v)`, tc.Value)
		t.Logf(`%#v (%%#v)`, tc.Value)
	}
}

func TestStruct(t *testing.T) {
	var list = []interface{}{
		article.Foo{
			Bar: 1,
			Baz: "Hello, WEB+DB PRESS!",
		},
		article.FooStringer{
			Bar: 1,
			Baz: "Hello, WEB+DB PRESS!",
		},
	}

	// 特筆すべきはarticle.Fooは%#vの際にquux（エクスポートされていないフィールド）
	// も出力されてしまうことと、article.FooStringerは%vも%#vも出力が完全に
	// カスタマイズできているということです。
	for _, v := range list {
		t.Run(reflect.TypeOf(v).String(), func(t *testing.T) {
			t.Logf("%v", v)
			t.Logf("%v", &v)
			t.Logf("%+v", v)
			t.Logf("%+v", &v)
			t.Logf("%#v", v)
			t.Logf("%#v", &v)
		})
	}
}
